(define-module (ui rd_freerdp ajax)
               :use-module (alterator ajax)
               :use-module (alterator woo)
               :use-module (alterator card-index)
               :export (init))


; Debug string (paste in for your needs).
;(format #t "XXX: ~S~%" rdp_profile_name)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; CREATE PAGE
;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


; Create profile form
(define (create-new-profile)
  (catch/message
    (lambda()
      (hide-all)
      ; Show form
      (form-update-visibility "new_profile_invalid" #f)
      (form-update-visibility '("new_profile_area") #t)
      ; Reset protocol name
      (form-update-value "new_profile_name"  ""))))


; Create new profile based on entered values
(define (create-new-profile-confirm)
  (let
    ((name (form-value "new_profile_name")))
    ; Test for correctness input	
    (if (regexp-exec (make-regexp (string-append "^" "" "$") regexp/extended) name)
      ; Empty name
      (and
        (form-update-value "new_profile_invalid_reason" "Profile name can't be empty")
        (form-update-visibility "new_profile_invalid" #t))
      ; Good name
      (and (catch/message
             (lambda () (woo-new "/rd"
                                 'name name
                                 'protocol "freerdp")))
           (form-update-visibility '("new_profile_area") #f)
           (select-profile name)))))

; Select just created profile
(define (select-profile name)
	(form-update-value "avail_profiles" name)
	(edit-profile))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; RDP
;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


; RDP init
(define (rdp-init)
  (form-value "name")
  (form-bind "rdp_reset" "click" edit-profile)
  (form-bind "rdp_save"  "click" rdp-write)
  (form-bind "btn-share-add" "click" rdp-share-add)
  (form-bind "btn-share-del" "click" rdp-share-del))

; Delete selected share
(define (rdp-share-del)
  (let ((rdp_profile_name (f-get-ci-value "avail_profiles"))
        (rdp_share (form-value "rdp-shares")))
    (and (not (equal? "" rdp_profile_name))
         (not (eq? rdp_share #f))
         (catch/message (lambda()
                          (woo "rdp" "/rd_freerdp/share_del"
                               'rdp_profile_name rdp_profile_name
                               'rdp_share rdp_share
                               'language (form-value "language"))))
         (form-update-enum "rdp-shares" (woo "rdp" "/rd_freerdp/list_shares" 'rdp_profile_name rdp_profile_name 'language (form-value "language"))))))

; Add new share
(define (rdp-share-add)
	(let ((rdp_profile_name (f-get-ci-value "avail_profiles"))
				(rdp_share_name (form-value "rdp-share-name"))
				(rdp_share_path (form-value "rdp-share-local-path")))
		(and (not (equal? "" rdp_profile_name))
				 (not (equal? "" rdp_share_name))
				 (not (equal? "" rdp_share_path))
				 (catch/message (lambda()
													(woo "rdp" "/rd_freerdp/share_add"
															 'rdp_profile_name rdp_profile_name
															 'rdp_share (string-append rdp_share_name ":" rdp_share_path)
															 'language (form-value "language"))))
				 (form-update-enum "rdp-shares" (woo "rdp" "/rd_freerdp/list_shares" 'rdp_profile_name rdp_profile_name 'language (form-value "language")))
				 (form-update-value "rdp-share-name" "")
				 (form-update-value "rdp-share-local-path" "")
				 )))

; Save RDP profile
(define (rdp-write)
  (catch/message (lambda()
                   (apply woo "rdp" "/rd_freerdp/write"
                          (form-value-list (append '("avail_profiles" "rdp_quality" "rdp_sound" "rdp_color_depth" "rdp_geometry") rdp-parameters)))))
  (edit-profile))

; Know RDP parameters
(define rdp-parameters '(
												 "rdp_profile_name"
												 "rdp_server"
												 "rdp_user_name"
												 "rdp_password"
												 "rdp_domain"
												 "rdp_rdp_shell"
												 "rdp_client_name"
												 "rdp_startup_path"
												 "rdp_publish_local_printers"
												 "rdp_disable_tls_encryption"
												 "rdp_disable_clipboard_synchronization"
												 "rdp_attach_to_console"
												 "rdp_fullscreen"
												 "rdp_audio_input_redirection"
												 ))

; Edit RDP
(define (rdp-edit rdp_profile_name)
  (form-update-visibility '("rdp_area") #t)
  (let* ((profile_info (woo "rdp" "/rd_freerdp/read" 'rdp_profile_name rdp_profile_name 'language (form-value "language"))))
    (form-update-enum "rdp_quality" (woo "rdp" "/rd_freerdp/list_quality" 'rdp_profile_name rdp_profile_name 'language (form-value "language"))) 
    (form-update-enum "rdp_sound" (woo "rdp" "/rd_freerdp/list_sound" 'rdp_profile_name rdp_profile_name 'language (form-value "language"))) 
    (form-update-enum "rdp_color_depth" (woo "rdp" "/rd_freerdp/list_color_depth" 'rdp_profile_name rdp_profile_name 'language (form-value "language"))) 
    (form-update-enum "rdp_geometry" (woo "rdp" "/rd_freerdp/list_geometry" 'rdp_profile_name rdp_profile_name 'language (form-value "language"))) 
    (form-update-enum "rdp-shares" (woo "rdp" "/rd_freerdp/list_shares" 'rdp_profile_name rdp_profile_name 'language (form-value "language"))) 
    (form-update-value-list '("rdp_quality" "rdp_sound" "rdp_color_depth" "rdp_geometry") profile_info)
    (for-each (lambda(p) (form-update-value p "")) rdp-parameters)
    (form-update-value-list rdp-parameters profile_info)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAIN MODULE
;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Confirm delete selected profile
(define (delete-profile-confirm)
	(hide-all)
	(let ((profile_name (f-get-ci-value "avail_profiles")))
				(form-update-value "delete_profile_name" profile_name)
				; Show form
				(form-update-visibility '("delete_profile_area") #t)))
           

; Delete profile
(define (delete-profile)
	(let ((profile_name (f-get-ci-value "avail_profiles")))
		(and 
			(catch/message (lambda() (woo-delete "/rd" 'profile_name profile_name 'language (form-value "language"))))
			(init-interface))))

; Interface initialization
(define (init-interface) 
	(catch/message 
		(lambda()
			; Hide all modules
			(hide-all)
			; Ask for available profiles
			(form-update-enum "avail_profiles" (woo-list "/rd_freerdp/avail_profiles" 'language (form-value "language"))))))

; Hide all
(define (hide-all)
	(form-update-visibility '("new_profile_area" "delete_profile_area" "delete_button" "rdp_area") #f))

; Edit profile with specific type
(define (edit-profile)
  ; Update list, keep previous value or set the first one, return value
  (f-update-ci-list "avail_profiles" "/rd_freerdp/avail_profiles")
  ; Get profile info
  (let* ((profile_name (f-get-ci-value "avail_profiles")))
    (hide-all)
    (form-update-visibility '("delete_button") #t)
    (rdp-edit profile_name)))

; Initialization routine
(define (init)
  ; Interconnect button with action 
  (form-bind "button_create_new_profile" "click" create-new-profile)
  (form-bind "button_create_new_profile_confirm" "click" create-new-profile-confirm)
  (form-bind "delete_button" "click" delete-profile-confirm)
  (form-bind "delete_profile_confirm" "click" delete-profile)
  (form-bind "delete_profile_cancel" "click" edit-profile)
  ; Selection profile lead to it editing
  (form-bind "avail_profiles" "change" edit-profile)
  ; Interface initialization
  (init-interface)
  ; Init modules
  (rdp-init))

